terraform {
  backend "azurerm" {
    resource_group_name  = "{{resource_group}}"
    storage_account_name = "{{storage_account_name}}"
    container_name       = "{{container_name}}"
    key                  = "{{tf_blob_name}}"
  }
}
