apiVersion: external-secrets.io/v1
kind: ClusterSecretStore
metadata:
  name: azure-keyvault
spec:
  provider:
    azurekv:
      authType: ServicePrincipal
      authSecretRef:
        clientId:
            name: azure-sp-credentials
            key: client_id
            namespace: external-secrets-system
        clientSecret:
            name: azure-sp-credentials
            key: client_secret
            namespace: external-secrets-system
      vaultUrl: https://{{keyvault_name}}.vault.azure.net
      tenantId: {{tenant_id}}