{
  "customer": "{{customer}}",
  "environment": "{{environment}}",
  "resource_group": "{{resource_group}}",
  "location": "{{location}}",
  "shloc": "{{shloc}}",
  "suffix": "{{suffix}}",
  "keyvault_name": "{{keyvault_name}}",
  "gh_repo_owner": "btenvs",
  "gh_btenvs_app_id": "{{gh_btenvs_app_id}}",
  "gh_btenvs_app_private_key_path": "{{gh_btenvs_app_private_key_path}}",
  "gh_btenvs_installation_id": "{{gh_btenvs_installation_id}}",
  "sp_client_id": "{{sp_client_id}}",
  "sp_client_secret": "{{sp_client_secret}}",
  "acr_username": "kubernetes",
  "acr_password": "GnHcg8KeltGiLbmv9WkDY6zp5RKWy+2dqcccGFtgcJ+ACRBzCt+i",
  "acr_registry_url": "beacontower.azurecr.io",
  "rancher_api_url": "{{rancher_api_url}}",
  "rancher_api_token": "{{rancher_api_token}}",
  "rancher_cloud_creds_name": "{{rancher_cloud_creds_name}}",
  "kubernetes_version": "v1.34.1+rke2r1",
  "port_list": ["30080/tcp", "30443/tcp"],
  "fault_domains": "1",
  "cluster_vnet_cidr": "10.0.0.0/16",
  "cluster_subnet_cidr": "10.0.1.0/24",
  "update_domain_count": 1,
  "os_image": "Canonical:0001-com-ubuntu-server-focal:20_04-lts-gen2:latest",
  "node_disk_size": 40,
  "node_storage_type": "Premium_LRS",
  "node_size": "Standard_D2s_v5",
  "node_count": 3
}