{
  "subscription_id": "{{subscription_id}}",
  "resource_group": "{{resource_group}}",
  "environment": "{{env_name}}",
  "customer": "{{customer}}",
  "suffix": "{{suffix}}",
  "keyvault_name": "{{keyvault_name}}",
  "github_owner": "btenvs",
  "github_app_id": "{{github_app_id}}",
  "github_app_private_key_path": "{{github_app_private_key_path}}",
  "github_installation_id": "{{github_installation_id}}",
  "kubeconfig_path": "{{kubeconfig_path}}",
  "keyvault_sp_id": "{{keyvault_sp_id}}",
  "keyvault_sp_secret": "{{keyvault_sp_secret}}",
  "tenant_namespace": "{{tenant_namespace}}",
  "auth_signin_flow_url": "",
  "auth_ropc_signin_flow": "b2c_1_ropc",
  "auth_b2c_tenant_name": "",
  "auth_b2c_default_app_id": "",
  "frontend_app_client_id": "",
  "frontend_app_client_secret": "",
  "acr_name": "{{acr_name}}",
  "acr_password": "{{acr_password}}",
  "acr_registry": "{{acr_registry}}",
  "location": "{{location}}",
  "shloc": "{{shloc}}",
  "ingress_eh_retention": 7,
  "timeseries_eh_retention": 7,
  "provider_iothubs": {{{provider_iothubs}}}
}